@part(cprog, root="cmtman.mss")
@chapter(Programming in C)
@section(Introduction)
This chapter is not about how to write in C Major, nor is it about how to
reallize a piece by Terry@Index(Terry Riley) Riley with a computer.  Rather,
it is about a programming language called ``C''.  Specifically, this chapter
is about how to use a somewhat twisted subset of the C language to make
music with CMT.  I hope to expand this chapter in the future, but for now,
it's going to look more like lecture notes than a complete guide to
programming@Index(programming).

@section(Writing a Program)
Let's say you want to write a program called ``foo'' that plays a couple
of notes.  First use the editor to make a file called ``foo.c'':
@begin(programexample)
edit foo.c
@end(programexample)
When you get into the editor, type the following program:
@begin(programexample)
/* a simple program */
#include "cmtprog.h"

mainscore()
    begin
        note(48, 100);
        note(50, 50);
        note(52, 50);
        note(53, 100)
    end
@end(programexample)
This program will play 4 notes in sequence@Index(sequence) by ``calling''
@i(note@Index(note)) 4 times.  You indicate what kind of note you want by
giving two @i(parameters) in parentheses after the word ``note''.  The
parameters for note specify pitch and duration.  The pitch parameter is 48
for middle C (C4), and goes up by one for each semitone.  Thus, the pitches
in this program are C4, D4, E4, and F4.  The second parameter is the
duration parameter, expressed in hundredths of a second.  Thus, the first
note will be 1 second long, the next two will be 0.5 seconds, and the last
will be 1 second.

A few more observations:  notice that the ``score'' consists of calls to
@i(note) preceded by a name, ``mainscore@Index(mainscore)()'', and is
bracketed by the words ``begin@Index(begin)'' and ``end@Index(end)''.  Also
notice that calls to note are separated by semicolons@Index(semicolons) (;).
All of these details are essential, and C is not very forgiving of even
trivial mistakes.  The first line of the program is called a
@i(comment@Index(comment)).  Any text between /* and */ is ignored by C.@:
You will find it useful to use comments to remind yourself what your program
does, when it was written, and so on.  The next line, beginning with
``#include@Index(include)'' tells the C compiler to read the file
``musiprog.h@Index(musiprog.h)'' from the system library.  This file
contains important definitions for music, so every program you write should
contain this line.

Unlike Adagio, C is a @i(case sensitive) language.  This means that
mainscore, MainScore, MAINSCORE, and MaInScOrE are all distinct identifiers.
Because of this, I suggest that you @i(never use upper case letters in C
programs), except in strings and comments.

To hear the results of this
program, you have to @i(compile) your program, then @i(link) it, and then
@i(execute) it.  The Lattice C compiler@Index(compiler) (invoked by
the command @i(cmtc)@Index(cmtc))
translates your program into a machine language form which is stored in a
file.  This file is then @i(linked@Index(link)), or combined with other
programs to produce an executable program file.  This file can then be
executed, meaning that the machine instructions in the  file are loaded into
working memory, and the computer then carries them out.  To compile your
program (called foo.c), type the following:
@begin(programexample)
cmtc foo.c
@end(programexample)
If your program has errors@Index(errors), the line number where the error
was discovered will be printed along with a brief description of the error.
You must fix these errors by editing your program and then compiling again.
When the compile phase and then the link phase completes without errors, you
can link your program by typing:
@begin(programexample)
cmtl foo
@end(programexample)
The command @i(cmtl@Index(cmtl)) is short for @i(cmt link).  Be sure
the last character is the letter l, not the numeral 1.  If no errors are
reported by the linker, your program is ready to run.  Otherwise, you will
have to fix your program with the editor, recompile, and relink.  To run
your program, type
@begin(programexample)
foo
@end(programexample)
To run your program again, you need not
recompile unless you change your program with the editor.

@section(Stopping a Program)
@Index(Stopping programs)
Sometimes you need to stop programs in progress before they finish
on their own.  You can type @c(ctrl)-C (hold down the @c(ctrl) key
and type ``C'' or @c(ctrl)-@c(break) to stop your program in most cases.
It is possible to write programs that cannot be stopped except by
turning off the computer, but these are unusual@foot{Typing @c(ctrl)-C
or @c(ctrl)-@c(break) sets a flag but does not terminate your program
immediately, because doing so might leave the MPU-401 interface in a
state that would cause problems later on.  Instead, the @i(gettime) function
checks the flag and carefully exits your program if the flag is set.  Since
@i(gettime) is called by almost every procedure that takes any significant
time to execute (e.g. @i(note), @i(rest), and @i(rest@ux(  )until)), it is
a pretty safe bet that typing @c(ctrl)-C will terminate any program.}

@section(Writing a Procedure)
Procedures@Index(procedure) allow you to give a name to a musical phrase.
The phrase can then be called from several places.  This saves your having
to retype all of the calls to @i(note) every time you want to hear the
phrase.  In the following example, there are two phrases, or procedures,
@i(up) and @i(down), which are called from @i(mainscore):
@begin(programexample)
#include "cmtprog.h"

mainscore()
    begin
        up();
        down();
        up();
        note(48, 100)
    end

up()
    begin
        note(48, 20);
        note(50, 20);
        note(52, 20);
        note(53, 20)
    end

down()
    begin
        note(55, 20);
        note(53, 20);
        note(52, 20);
        note(50, 20)
    end
@end(programexample)
Notice that @i(up) and @i(down) obey the same rules as @i(mainscore):
procedure names are followed by a pair of parentheses and the word
@i(begin).  Then there is a list of calls to @i(note) or other procedures
separated by semicolons.  While @i(note) has two parameters, @i(up)
and @i(down) each have zero parameters.  You indicate the absence of
parameters when you call @i(up) or @i(down) by not putting anything
between the open and close parentheses.  However, you @i(must) always
type the parentheses after the name of any procedure you call.
Finally, the sequence of calls is ended by the
word @i(end).

When this program is run, the computer will do what @i(mainscore) says
to do.  The first call is to @i(up), so the computer finds the definition
of @i(up) and does what @i(up) says to do.  In this case, @i(up) plays
four notes (C, D, E, F).  Now that @i(up) is finished, @i(mainscore)
continues by calling @i(down), which in turn plays (G, F, E, D).
When @i(down) returns, @i(mainscore) continues with another call to
@i(up), and again @i(up) will play C, D, E, F.@:  Finally, @i(up)
returns and mainscore plays a C.@:  At this point the @i(mainscore)
program is finished.

@section(Repeats)
Repetition@Index(repetition)@Index(repeats)
 is important in both programming and
music.  To repeat a phrase, a special ``repeat'' construct is provided.
Consider the following example:
@begin(programexample)
#include "cmtprog.h"

mainscore()
    begin
	repeat(i, 5)
	    note(48, 30);
	    note(60, 30)
	endrep
    end
@end(programexample)
Look at the two calls to @i(note) above.  By themselves, these calls would
play the phrase C4 C5.  The repeat construct consists of the form
@example[repeat(@i(counter, howmany)) @i(phrase) endrep]
where @i(counter) is just the
name you want the computer to give to a variable number that keeps track
of which repeat is being played  (more about this later), @i(howmany) is
the number of repeats to take, and @i(phrase) is a sequence of
calls to @i(note) or any other procedure.
The example above will play the phrase C4 C5 C4 C5 C4 C5 C4 C5 C4 C5 because
we wrote 5 for the number of repeats.  If you like, you can repeat up to
about 30,000 times.

The phrase could just as well have been calls to other procedures.  The
next example uses @i(repeat) to play C4 D4 E4 F4 three times, using the
@i(up) procedure used earlier.
@begin(programexample)
#include "cmtprog.h"

mainscore()
    begin
	repeat(i, 3)
	    up()
	endrep
    end

up()
    begin
        note(48, 20);
        note(50, 20);
        note(52, 20);
        note(53, 20)
    end
@end(programexample)

@section(Conditions)
Computer programs are made more flexible by the use of
conditionals@Index(conditionals), that is, the ability to test a condition
and act upon the result of the test.  For example, suppose you want to
repeat a phrase, but you want the phrase to have a first@Index(first ending)
and second ending@Index(second ending).  In other words, the first time
through you want the end of the phrase to be played one way, and the second
time through, you want the ending to be played another way.  How would you
do this using just repeats and procedures?  The following program uses a new
construct (the @i(when@Index(when)) construct) that makes this programming task easy:
@begin(programexample)
#include "cmtprog.h"

mainscore()
    begin
	repeat(i, 2)
	    note(60, 30);
	    note(59, 15);
	    note(57, 15);
	    note(55, 15);
	    note(53, 15);
	    note(52, 15);
	    when i == 1 then
		note(53, 15);
		note(55, 120)
	    otherwise
		note(50, 15);
		note(48, 120)
	    endwhen
	endrep
    end		
@end(programexample)
This is a program with a @i(repeat) construct, but notice that the last
part of the repeated phrase is of the form:
@example[when @i(condition) then @i(phrase1) otherwise @i(phrase2) endwhen]
The computer interprets this construct as follows:  whenever a @i(when)
is encountered, the computer evaluates the following condition.  In this
case the condition is ``i == 1'', which is true when the repeat counter (i)
is equal to one (the first time through) and false when the repeat counter
is not equal to one@foot(The double equal symbol ``=='' is interpreted by
C to mean ``is equal to''.  A single equal symbol ``='' is called the
@i(assignment) operator, and should never be used unless you know what you
are doing.  Even experienced programmers occasionally slip and write ``=''
when they mean to write ``==''.  It is a good idea to use the editor to
search for all occurrences of ``='' to make sure your program is correct.).
If the condition is true (which happens the first time through in this
example), the phrase following @i(then@Index(then)) is performed up to the
word @i(otherwise@Index(otherwise)), and the phrase between @i(otherwise)
and @i(endwhen@Index(endwhen)) is skipped.  If the condition is false, the
phrase between @i(then) and @i(otherwise) is skipped, and the phrase between
@i(otherwise) and @i(endwhen) is performed.  As usual, these phrases can be
calls to other procedures like @i(up) and @i(down).

The @i(when) construct is used to select between two
alternatives@Index(alternatives) where the selection is based on a
condition.  Useful conditions are:
@begin(programexample)
a == b -- true if a is equal@Index(equal to) to b
a > b  -- true if a is greater than@Index(greater than) b
a < b  -- true if a is less than@Index(less than) b
a >= b -- true if a is greater than or equal to b
a <= b -- true if a is less than or equal to b
a != b -- true if a is not@Index(not equal) equal to b
@end(programexample)
Here, @i(a) and @i(b) stand for repeat counters (like i) or numbers (like 0,
1, @i(etc).).  Like the @i(repeat) construct, the @i(when) construct can be
used anywhere in a phrase.  The following example plays a phrase three
times.  On the second time through, the middle of the phrase is extended.
The procedures p1, p2, and p3 are not shown.
@begin(programexample)
#include "cmtprog.h"

mainscore()
    begin
	repeat(i, 3)
	    p1();
	    when i == 2 then
		p2()
	    otherwise
	    endwhen
	    p3()
	endrep
    end
@end(programexample)
In this example, the phrase between @i(otherwise) and @i(endwhen) is
nonexistent.  That means that on the first and third times through,
the complete phrase will be equivalent to p1(); p3().  On the second time
through, the phrase  will be equivalent to p1(); p2(); p3().

@section(Parameters)
It's time to confess:  there is really no difference between
@i(mainscore), @i(note), @i(up), and @i(down).  They are all procedures,
and every procedure has a list of @i(parameters@Index(parameters)), which
can serve to modify its behavior.  For example, the parameters to @i(note)
tell the note procedure what pitch and duration to use.  You can easily
write your own procedures that have (more than zero) parameters.  Study the
@i(newup) procedure below, which plays C4 D4 E4 F4.  The procedure has one
parameter, called @i(dur), which determines the duration of each note:
@begin(programexample)
#include "cmtprog.h"

mainscore()
    begin
	newup(25);
	newup(50)
    end;

newup(dur)
    begin
	note(48, dur);
	note(50, dur);
	note(52, dur);
	note(53, dur);
    end
@end(programexample)
When @i(mainscore) is played, it first calls @i(newup) with the parameter
25.  The computer will then find the definition of @i(newup) and notice that
the parameter is to be named @i(dur).  Now, whenever the computer finds the
name @i(dur) within @i(newup), it will substitute @i(dur)'s value (25).
Thus, the duration specified for all of the calls to @i(note) will be
25, or one quarter second.

After the fourth note is played, the computer returns to @i(mainscore), where
the next thing in the sequence is another call to @i(newup).  But this time,
the parameter is 50.  The computer goes through the same steps as before: it
finds the definition of @i(newup), associates the value of 50 with @i(dur),
and substitutes 50 for @i(dur) throughout the definition of @i(newup).
The result is that the four notes (C4 D4 E4 F4) are now played with durations
of 50, or one half second.

Parameters may be named with any string of letters, just like procedures.
It is a good idea to use @i(mnemonic) names, that is, names that remind you
of their purpose.  It makes no difference to the computer, but when you
start writing large programs, you will find that it is important to 
make programs readable and understandable.

@p(Important:)  When you use parameters, the number of parameters in the
definition @i(must) match the number of parameters in the call.  The order
of parameters in the call determines which parameter gets which value.
Neither of these properties are checked by the C compiler, so very strange
things will happen if you make a mistake here.

@section(Producing Chords)
@label(pnote-sec)
The C language is called a sequential language because programs are executed
in sequence, performing only one operation at a time.  This is a big
limitation for music, and the next chapter will present some solutions to
the problems of using C.  In the meantime, there is a fairly simple way to
get two notes sounding at once.  The procedure @i(pnote@Index(pnote)) is
just like @i(note) except that @i(pnote) does not wait for the specified
duration.  Instead, it @i(schedules) the end of the note to happen in the
future but returns immediately without waiting.  The following procedure
plays a minor chord@Index(chord) with the specified tonic and duration:
@begin(programexample)
minor(tonic, duration)
    begin
	pnote(tonic, duration);
	pnote(tonic + 3, duration);
	note(tonic + 7, duration)
    end
@end(programexample)
The first two notes of the chord are played using @i(pnote).  Since
@i(pnote) returns immediately, all three notes start very close
to the same time.  The third note uses the @i(note) routine, which will
delay for @i(duration) before returning.  Thus the @i(minor) procedure
will also take @i(duration) before returning.

@section(Low-level Procedures)
Up until now, we have concentrated on writing programs that control the
high level structure of your music.
By now, I hope you are beginning to enjoy some of the power available to you
as a computer programmer.  Now, it is time to learn about the lower levels
of control, which concern direct control over the synthesizer.  I mentioned
earlier that @i(note) is just a procedure.  Here it is:
@begin(programexample)
note(pitch, duration)
    begin
	midi@ux( )note(1, pitch, 100);
	rest(duration);
	midi@ux( )note(1, pitch, 0);
    end
@end(programexample)
The @i(note) procedure first uses the procedure @i(midi@ux(  )note) to start
a note.  The parameters are the MIDI channel number (1),  the pitch, and
the key velocity (100).

The @i(rest) procedure stops the program for the length of time
specified by @i(duration).  This sustains the note.

The third procedure call is another call on @i(midi@ux(  )note).  The
key velocity of zero (the third parameter) indicates to turn the note
off.

Incidentally, the Adagio program also uses the same @i(midi@ux(  )note)
procedure.
You can find the C programs that make up
Adagio in the directory ``\cmt''.


@subsection(Other Procedures and Functions)
A complete list of music functions and procedures
are listed in the file @i(\cmt\mpu.c)
and in Appendix @ref(mpu-app).
A summary of some useful ones, as well as some
useful C procedures are given below.  Italicized parameters stand for
numbers that you supply when you call the function.
@begin(description)
printf@Index(printf)(@i["this is a string\n"]);@\writes @i["this is a
string"] on the screen when called.  The two characters \n should be
included after the text you want written.  This tells the computer to
``write'' a newline after writing the line of text.

rest@Index(rest)(@i[duration]);@\does nothing for @i(duration) (expressed in
hundredths of seconds).

getkey@Index(getkey)(@i[waitflag]);@\gets a keyboard event.  Returns a the
key number (numbered starting at 0) when a key is pressed, and the key
number plus 128 when a key is released.  If @i(waitflag) is not zero,
@i(getkey) will wait for a key to be pressed.
If @i(waitflag) is 0, @i(getkey) may return -1 to
indicate no key has been pressed.  Example return values and their
meaning are -1: no key was pressed, 48: middle C was pressed, 176: middle
C was released (176 = 128 + 48).

midi@ux(  )note@Index(midi@ux(  )note)(@i(channel, pitch, velocity));@\sends a MIDI note-on command.
The parameters are the MIDI channel (from 1 to 16),
the key number (from -12 to 115), and the key velocity (from 0 to 127).
If the velocity is 0, then the note is turned off.

midi@ux(  )program@index(midi@ux(  )program)(@i(channel, program));@\changes
the synthesizer program (preset) to the one indicated in the second
parameter.  The first parameter is the MIDI channel.

random@Index(random)(@i[low, high]);@\a function whose value is a random
number between @i(low) and @i(high) inclusive.

@end(description)
@section(Command Line Options)
@begin(itemize)
-debug (or -d): turn on debugging code.

-help: print a list of command line options.

-miditrace (or -m): turn on MIDI byte trace.

-trace (or -t): turn on music operation trace.

-block: turns off the MPU-401 MIDI THRU function so that the input
is not passed to the output.

-tune @i(filename): tells Adagio to use @i(filename) to specify
tuning.
@end(itemize)

@subsection(Examples)
Assuming you have compiled a program named @i(foo),
@begin(programexample)
foo -help
@end(programexample)
will print a list of command line options.

To run @i(foo) with a tuning specified in @i(myscale.tun), with
a printout of music operations as they are performed:
@begin(programexample)
foo -tune myscale -t
@end(programexample)

@section(Conclusions)
Becoming a master programmer is like mastering an instrument; it takes
years of hard work.  Fortunately, you can make a lot of good computer
music without a degree in computer science@footnote(Of course, you @i(can)
make a lot of @i(bad) computer music @i(with) a degree in computer science.).
If you are confused about some point or about how
to implement an idea, ask someone for help.  It is often useful to write
small programs to test out ideas before you write a magnum opus.  Write
small procedures and test them before putting them into a large confusing
program.

If you want to learn more about programming,  consider taking a computer
science course if you haven't already.  Your instructors can suggest
textbooks if you want to study more independently.
  There are many programming concepts that I have not
even touched in this quick introduction.

I designed this system so that students could have all the flexibility
offered by programming in order to create new music.  Phil
Miller@Index(Miller, Phil) showed me how to distill the important concepts
to the point where basic programming skills can be developed in a few weeks.
Now it's up to you:  let there be music!
