@part(dos, root="cmtman.mss")
@chapter[DOS Utilities]
Before describing music software that comes with the CMU MIDI Toolkit,
some DOS commands and utility programs are described.  Three programs
are provided with CMT to help make your life easier.  These are @i(make),
@i(ls), and @i(grep).  A section on backing up files, intended for novice
users, is also included in this chapter.

@section(make)
@index(make)
The @i(make) program is based on the Unix@foot(Unix is a trademark of
A.T.& T. Bell Laboratories) program of the same name, and is documented
in the file @i(make.man).  The purpose
of @i(make) is to figure out what has to be recompiled when you edit
a file.  A file called @i(makefile) contains a list of file dependencies
and a command that specifies how to recompile a dependent file.  The
@i(make) program figures out what files have changed by examining their
``last modified'' times and dates.  Therefore, if you use @i(make), you
must always tell DOS the time and date when you turn on your machine.
An example of an entry in @i(makefile) describing a dependency is
@index(makefile)
@begin(example)
ADOBJ = adagio.obj phase1.obj phase2.obj adaspace.obj noteoff.obj
MUSICLIB = cmt.lib

adagio.exe : #(ADOBJ) #(MUSICLIB)
	link #(COBJ) #(ADOBJ),adagio,,#(LIBS)
@end(example)
In this example, @i(ADOBJ) is a symbol that is defined in the first line
to be a list of files.  The line with the colon (:) says that @i(adagio.exe)
depends upon everything on the right of the colon, that is, all of
the object files represented by @i(ADOBJ) and also the file @i(cmt.lib),
represented by @i(MUSICLIB).
If @i(adagio.exe) does not exist or is older than one of the files it
depends upon, it should be remade by executing the lines after the
line stating the dependency.  In this case, there is only one command
(starting with the word @i(link)) needed.

The file @i(makefile), from which this example was taken, is
provided with CMT and contains a complete list of
CMT file dependencies.  It is a good reference if you have questions
about how to write a @i(makefile).

To use @i(make), simply give it the file you want to make up-to-date.  For
example,
@begin(example)
make adagio.exe
@end(example)
will make @i(adagio.exe) up to date.  To make sure the entire CMT
is up-to-date, you can type
@begin(example)@index(everything)
make everything
@end(example)
Note: the @i(makefile) provided with CMT assumes you are in the
directory with all the CMT source files when you run @i(make).
To clean up leftover files, type:@index(clean)
@begin(example)
make clean
@end(example)
This will not remove .OBJ files.  You can get rid of unneeded .OBJ
files by typing:@Index(cleanobjs)
@begin(example)
make cleanobjs
@end(example)

@section(ls)
@index(ls)
The @i(ls) program is like the DOS @i(dir) command, except it lists
file names only (no size and date information), and the names are
formatted into columns in alphabetical order.  The current version
of @i(ls) does not work with large directories, although this may be
fixed by the time you read this.

@section(grep)
@index(grep)
The @i(grep) program searches one or more files for a text string.  If
the string is found, the line containing the string is printed. @i(grep)
is indispensable for reading someone else's code.  For example, to find
out where @i(fileopen)@index(fileopen)
is declared (as well as where it is used), type
@begin(example)
grep fileopen *.c
@end(example)
in the CMT source code directory.

@section(Backing up files)
@index(files)
@index(saving files)
@index(backing up files)
@index(floppy disks)
@index(flexible disks)
@index(files)
@p(Note:) 
This section is intended for new users of the CMU Computer Music Studio,
but it is worth reading if you are inexperienced with personal computers.

The IBM XT system has a built-in disk that can store about 10 million
characters of information.  Unfortunately, that may not be enough to meet the
demands of all of the studio users.  Fortunately, there is an alternative way
to store information called the floppy disk.
Floppy disks are also a good way to make copies of your work for safe keeping.
@i(There is always a small chance that all of your data will be lost from
the Studio computer.)  To protect yourself, you should keep a
backup@Index(backup) copy of any critical files on a floppy disk.  Floppy
disks are thin magnetic disks that come in a protective sleeve and insert
into the slot in front of the IBM XT.

To make a backup copy@Index(copy) of your directory,
@begin(enumerate)
put a floppy disk into the disk drive slot and push down the door

if you have not used the disk before, type 
@begin(example)
format a:
@end(example)
This will @i(completely erase) your disk, so only do this if the disk is
new or there is nothing on the disk you want to keep.

type
@begin(example)
copy *.* a:
@end(example)
@end(enumerate)
This will copy all files from your directory to the floppy disk.  If
there is a file on the floppy disk whose name is the same as a file
in your directory, then it will be erased and replaced.  For example,
if you copied a file called @i(tune.gio) to your floppy disk last
week, then edited @i(tune.gio), and now you back up you directory,
then you will lose last week's version of @i(tune.gio) from your
floppy disk.  It will be replaced by the current version.

To copy individual files, type
@begin(example)
copy @i(filename) a:
@end(example)
where @i(filename) is the file you want to save on floppy.  To recover
an individual file from floppy disk to your directory, type
@begin(example)
copy a:@i(filename)
@end(example)



