@part(mpu, root="cmtman.mss")
@appendix(The MIDI Interface)
@label(mpu-app)
This appendix contains documentation for calls in the module @i(mpu.c).
Type @i(boolean) is an @i(int) that takes on values of true or false, and
type @i(byte) is an unsigned char.

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


exclusive@Index(exclusive)(onflag)
@\boolean onflag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
This procedure tells the MPU-401 to read exclusive messages into the buffer
provided by a previous call to @i(midi@ux(  )buffer).
@i(onflag) is set to true to receive MIDI exclusive data, otherwise 
it is set to false.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


boolean getbuf@Index(getbuf)(waitflag, p)
@\boolean waitflag;
@\byte * p;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
@i(getbuf) copies 4 bytes of data from @i(buffer) (the MIDI input
buffer filled by the interrupt handler in @i(aintr.asm)) to @i(*p).  It
will wait for buffer to become nonempty if and only if @I(waitflag) is true.
It returns true if data was written to @i(*p), and
false if data was not written to @i(*p).
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


int getkey@Index(getkey)(waitflag)
@\boolean waitflag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
@i(getkey) tries to read a key.  It returns the
key number of the key which has been depressed, or the key number plus
128 of a key which has been released. 
It returns -1 if @i(waitflag) is false and no key has been pressed.
If @i(waitflag) is true this routine will block until a key is pressed.
Since @i(getkey) uses @i(getbuf) (see above), 
non-key events (e.g. pitch bend) may be read and discarded by @i(getkey).
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


long gettime@Index(gettime)()
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Return the time in 100ths of seconds since the last call to
@i(musicinit) or @i(timereset).
@end(text)
@end(group)


@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


l@ux(  )rest@Index(l@ux(  )rest)(time)
@\long time;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
@i(l@ux(  )rest) waits until the amount of time specified by
@i(time) has elapsed.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


l@ux(  )restuntil@Index(l@ux(  )restuntil)(time)
@\long time;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
@i(l@ux(  )restuntil) waits until the specified @i(time) has been reached
(absolute time as returned by @i(gettime)).
@end(text)
@end(group)


@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


metronome@Index(metronome)(onflag)
@\boolean onflag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
@i(metronome) enables (true) or disables (false) MPU-401 metronome function,
as indicated by @i(onflag).
This procedure must be called before @i(musicinit).  The metronome is not
started until @i(musicinit) is called.
@end(text)
@end(group)


@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )bend@Index(midi@ux(  )bend)(channel, value)
@\int channel;
@\int value;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Sends a MIDI pitch bend message; @i(channel) is
the MIDI channel on which to send data, and @i(value) is
the pitch bend value from 0 to 16383.  The value 8192 is the
value representing no pitch bend; smaller values bend pitch down,
and higher values bend pitch up.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


int xbuffhead, xbufftail;

int midi@ux(  )buffer@Index(midi@ux(  )buffer)(buffer, size)
@\byte * buffer;
@\int size;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
@i(midi@ux(  )buffer) tells interrupt routine to store system exclusive messages in
@i(buffer).  The largest power of 2 bytes less than @i(size) will be
used (a power of 2 is used so that masking can be used to cause
wrap-around).  @i(xbuffhead) and @i(xbufftail) will be initialized to zero,
and @i(xbufftail) will be one greater than the index of the last
system exclusive byte read from mpu401.
@i(midi@ux(  )buffer) returns 
false if @i(size) is less than 16 or @i(buffer) is NULL, otherwise true.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )cont@Index(midi@ux(  )cont)(onflag)
@\boolean onflag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
@i(midi@ux(  )cont) enables (true) or disables (false) continuous control
info from the MPU-401 to the host, depending on the value of @i(onflag).
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )ctrl@Index(midi@ux(  )ctrl)(channel, control, value)
@\int channel;
@\int control;
@\int value;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Sends a MIDI control change message; @i(channel) is
the MIDI channel on which to send data, @i(control) is the control number,
and @i(value) is the control value.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )exclusive@Index(midi@ux(  )exclusive)(msg)
@\byte * msg;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Sends a MIDI exclusive message; @i(msg) is a pointer to a MIDI
exclusive message, terminated by 0xF7.
@end(text)
@end(group)


@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )note@Index(midi@ux(  )note)(channel, pitch, velocity)
@\int channel;
@\int pitch;
@\int velocity;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Sends a MIDI note-on request out; @i(channel) is the MIDI channel on which
to send data, @i(pitch) is the MIDI pitch code, and @i(velocity) is
the velocity with which to sound it (0 means release).  If @i(read@ux(  )tuning)
has been called to define a tuning and @i(velocity) is not zero,
then this procedure sends a pitch
bend followed by a note-on message.  The pitch bend and the note-on pitch
are found by table lookup, using @i(pitch) as the table index.
For zero @i(velocity), the table lookup is done to get the pitch for a
note-on message with zero @i(velocity).  No pitch bend message is sent
since zero velocity means note-off.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )program@Index(Preset)@Index(Program change)@Index(midi@ux(  )program)(channel, program)
@\int channel;
@\int program;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Sends a program change request; @i(channel) is
the channel on which to send the MIDI program change request, and
@i(program) is the program number to send (this number is
decremented by 1 before	being sent as MIDI data, so values should
be in the range of 1 to 128.)
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )thru@Index(midi@ux(  )thru)(onflag)
@\boolean onflag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Enables (true) or disables (false) MIDI thru info from
MPU-401 to host, depending on the value of @i(onflag).
The default is enabled, but the command line switch -block will
cause it to be disabled initially.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


midi@ux(  )touch@Index(midi@ux(  )touch)(channel, value)
@\int channel;
@\int value;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Sends a MIDI after touch message; @i(channel) is the
MIDI channel on which to send data, and @i(value) is
the after touch value.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


mpu@ux(  )error@ux(  )check@Index(mpu@ux(  )error@ux(  )check)()
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Reports any errors originating in the interrupt handler by printing them
on stdout.  If you do not call this routine, you do not find out about
the errors.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


mpuexists@Index(mpuexists)(flag)
@\boolean flag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
If argument is false, this indicates no MPU-401 is on the machine, so
simulate the mpu-401 (for debugging only).
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


read@ux(  )tuning@Index(read@ux(  )tuning)(filename)
@\char *filename;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Reads tuning information from @i(filename) and enables pitch bend to
be sent by @i(midi@ux(  )note) to effect detuning.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


musicinit@Index(musicinit)()
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Initializes the MPU-401 device driver and resets the MPU-401.
The clock (see @i(gettime)) is reset to zero.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


musicterm@Index(musicterm)()
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Cleans up; disables MPU-401 interrupts; resets MIDI devices.
Call this upon finishing execution.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


int random@Index(random)(lo, hi)
@\int lo, hi;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Returns a pseudo-random number between @i(lo) and @i(hi) inclusive
(@i(lo) <= result <= @i(hi)).
This routine is reasonably fast, but it should not be trusted
to be very ``random''.
@end(text)
@end(group)


@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


timereset@Index(timereset)()
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Resets the time on the MPU-401.	 The time returned by @i(gettime)
is reset to 0.
@end(text)
@end(group)


@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


trace@Index(trace)(flag)
@\boolean flag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Turns tracing on (if @i(flag) is true) or off (if @i(flag) is false).
Calls to @i(mpu.c) are traced.
@end(text)
@end(group)

@begin(group)
@begin(format)
@tabclear
@tabset(0.2 in)


tracemidi@Index(tracemidi)(flag)
@\boolean flag;
@end(format)
@begin(text, leftmargin +0.5 in, indent 0 in)
Turns MIDI tracing on (if @i(flag) is true) or off (if @i(flag) is false).
MIDI output bytes are printed.
@end(text)
@end(group)

