@part(next, root="studioman.mss")
@chapter(What next?)
A number of improvements remain to be made to the CMU MIDI Toolkit.  To
encourage others to make these improvements, CMT is distributed with source
code and documentation.  In addition, the Center for Art and Technology at
Carnegie Mellon University will help coordinate and integrate changes to
this software.  If you develop a new tool or enhance an existing one, you
can incorporate it into CMT by sending it to me (Roger B. Dannenberg) at the
Center for Art and Technology.  Provided the changes or additions are
consistent with the goals of CMT, I will integrate them into the next
release and redistribute your code to other users.  Thus others will benefit
from your efforts as I hope you have benefited from mine.

The following sections outline changes I would like to see in the
CMU MIDI Toolkit.  If you decide to tackle one of these, it might be
a good idea to call me at (412) 268-3827 or write to find out if anyone
else is working on the same thing.

@section(Microsoft C conversion)
The current version of CMT uses the Lattice@Index(Lattice C) C compiler.
Work is in progress to make CMT compatible with both Lattice and
Microsoft@Index(Microsoft C) C.

@section(Large memory model conversion)
@index(memory model)
@index(small memory model)
@index(large memory model)
The most serious problem with CMT at the moment is that it was designed
for use with the ``small memory model''.  This means that only 16 bits
are used to address code and data, so not all of the available memory
can be used.  This is a problem that limits Adagio to less than 4 
thousand notes.  To use another memory model, one would have to recode
some of the assembler routines (particularly @b(aintr.asm)) to get
the right values into the segment registers.

@section[Multiple Midi Interfaces]
@index(Multiple Midi Interfaces)
Another limitation of CMT is that it supports only one MPU-401.  There
are two reasons for using more than one.  First, you might want to have
more than one MIDI input to the system.  A keyboard like the KX88 will
``mix'' the data it generates with its MIDI Input to form the MIDI Output.
Thus, you can get input from a KX88 and at least one other device by
connecting them in series.  However, most MIDI devices cannot ``mix''
MIDI data, so one might want to have several MIDI interfaces to get input
from several controllers.  Notice that input data can be ``mixed'' internally
by the IBM PC, and there is no reason to distinguish which physical input
is the source of a given piece of data.

The second reason for multiple interfaces is to overcome the maximum of 16
MIDI channels or to obtain higher speed.  A reasonable thing to do in this
case is to implement ``logical'' MIDI channels.  Logical
channels 1 through 16 would correspond to MIDI channels 1 through 16 on the
first MPU-401.  Logical channels 17 through 32 would correspond to MIDI
channels 1 through 16 on the second MPU-401, and so on.  This presents no
problems for Adagio, but there is a lot of CMT software that assumes only
4 bits are needed to store a channel number.  Implementing logical channels
is more work than the mixing idea described in the preceding paragraph.

@section[Improve Adagio compilation (sorting) speed]
Currently, Adagio keeps a linked list of notes and control changes
sorted by start time.  If scores are roughly in time order, the
performance is good.  However, if a score is constructed by concatenating
several files created using @i(transcri) or @i(record), then notes
are not in order and Adagio can spend several minutes loading a score.
I believe this can be solved by two transformations.  First, change the
insert routines to insert notes in forward rather than reverse time order
and take out the code that reverses the final score.  Second, keep a pointer
to the last point at which a note was inserted.  When you go to insert a
note, try searching forward from the saved pointer.  Only if this fails
should you search from the beginning of the list.  The resulting program
will still perform badly on perverse scores, but the expected performance
should be quite good on normal scores.

@section(Multi-track recording)
@index(multi-track recording)
@index(recording)
Not everyone has enough equipment to play an entire score, and
not all scores will fit into the available memory.  A solution to
these problems is to use a multi-track recorder and to record a
composition one track at a time.  To do this, one must be able to 
enable one Adagio voice at a time.  I suggest adding an option to the
Adagio (and @i(Record)) command line of the form
@begin(example)
-v 1,5
@end(example)
which would mean play only voices (MIDI channels) 1 and 5.  Notes
and control events on other channels would not be saved in memory
for playing, so this facility would help get around memory restrictions.

For long works and pieces where close synchronization@Index(synchronization)
of parts is important, it is essential to have some means of synchronizing
Adagio output with a tape recorder.  The MPU-401 has a
tape-sync@Index(tape-sync) input and output which could be used for this
purpose.  Probably, a command-line switch (``-sync'') should be read by
@b(mpu.c).  When the switch is present,
@b(mpu.c) should tell the MPU-401 to synchronize to the tape input.
Notice that CMT has optional code to derive time from the
internal IBM-PC timer rather than the MPU-401.  If that is the case,
then the ``-sync'' switch should either cause @b(mpu.c) to revert back
to the MPU-401 or there should be some way to cause MPU-401 clock
input data to resynchronize the internal IBM-PC timer.

@section(MIDI Continuous Controls)
Adagio supports the notion of continuous controls through its control
change commands, but these are presently very limited.  Three enhancements
should be made.  First, it should be possible to program how control change
commands in Adagio are mapped into MIDI control change commands.  For
example, one might want to use the X command to send breath-controller
values.  This could be implemented by having an optional control specification
file, just as there is now an optional tuning specification file.  The
same file format should be used by @i(transcri) and @i(record) to
determine how Adagio scores are produced from MIDI input.  
One could use this same facility to tell @i(record) and @i(transcri) to
ignore certain control information.  For example, if no command is
assigned to control aftertouch, then aftertouch data would be ignored in
the recording (transcription)  process.


A second change would be to add new commands to Adagio to allow users
to specify smooth, continuous changes without a huge amount of typing.
This could be done by implementing a new special command.  For example,
@begin(example)
!ramp x0 x100 q. 
@end(example)
would mean change the X control smoothly from 0 to 100 in the time of
a dotted quarter note.  This command  would be translated into a
single Adagio event and the stream of control change data would be
computed from the event data in real time.  Thus, in addition to 
save typing, the scheme would save on memory space wherever these
special commands could be used in place of many control change commands.

Finally, it would be very interesting to look into using control changes
to modify MIDI programs in real-time.  For example, you can put a DX7 in
a mode where the inharmonicity of a tone can be controlled by a slider on
the DX7 control panel.  By using system exclusive messages, one can control
the inharmonicity by computer.  The missing link here is to make the
control specification file flexible enough that one can use Adagio to
send system exclusive messages as well as conventional control change
commands.  It might be helpful to add yet another command to Adagio
to send system exclusive messages directly.

@section(Interacting with Moxc programs)
@index(debug)
Every Moxc program I write has a big @i(asciievent)@Index(asciievent)
routine that calls different routines and sets values depending upon
characters that get typed at run time.  It should be possible to write
a routine that could read the map file produced by the linker and
figure out where variables and procedures are located.  Then, a user
interface could be designed such that you could type
@begin(programexample)
seti foo 2006
call bar 100 572
@end(programexample)
in order to set the integer @i(foo) to 2006 and then call procedure bar
with parameters 100 and 572.

@section(Code to play sequences from Moxc)
It is hard to write Moxc programs that play sequences of notes, and
it would be nice to be able to write sequences in Adagio and trigger
them in real time Moxc programs.  I designed a module for this and wrote most
of the code, but I have not tested anything yet.  The idea is to
use a modified Adagio to read scores and transform them into data files
that can be input easily without resorting to a full-blown Adagio
translator.  Routines are then provided to read and play
these sequences.  Several parameters including transposition and rate
can be changed in real-time to modify the performance of the sequence.
The interface is as follows:
@begin(description)
seq@ux(  )read(name, filename);@\reads the sequence in file @i(filename).
Sequences are named internally by small integers, and @i(name) is the
integer to associate with this sequence.

seq@ux(  )play(name);@\plays sequence associated with the integer @i(name).

seq@ux(  )transpose(name, transposition);@\sets the transposition of the
sequence associated with @i(name) to @i(transposition).

seq@ux(  )rate(name, rate);@\sets the rate of the sequence associated with
@i(name) to @i(rate).  A rate of 100 gives the normal rate as specified
in the Adagio file that generated the sequence.

seq@ux(  )cycle(name, flag);@\sets the cycle flag of the sequence associated
with @i(name) to @i(flag).  If the flag is non-zero, then the sequence
will repeat (cycle) indefinitely.

seq@ux(  )pause(name, flag);@\sets the pause flag of the sequence associated
with @i(name) to @i(flag).  If the flag is non-zero, the sequence performance
will be suspended until the flag is set to zero.

seq@ux(  )at@ux(  )end(name, procedure);@\will cause the routine named by
@i(procedure) to be called at the end of the performance of the sequence
associated with @i(name).
@end(description)
This implementation of this interface is quite compact.  A drawback
is that there is no provision for recording sequences in real-time
within the context of Moxc, so some more thinking and 
experimentation is probably in order.  It has also been suggested that
one should be able to combine rhythm from one sequence with pitches from
another.

@section(A player module)
Currently, Adagio and Record are very similar.  The main difference
is that in phase 2, in which Adagio plays a score, Record plays a score
@i(and) calls on @b(record.c) to record MIDI input.  It would be nice
to design a module @b(player.c) that both Adagio and Record could call
on to play a score.  This would eliminate a lot of redundant code and
would insure consistency between Adagio and Record when changes are made.

For example, one might want to allow users to transpose scores, change
the playback speed, and make other changes without editing the score.
At present these sorts of changes would have to be made in both @b(phase2.c)
for Adagio and @b(trans2.c) for Record.

@section(User-defined note routines for Adagio)
Not only would it be nice to be able to invoke Adagio-defined sequences from
Moxc, it would also be nice to be able to invoke Moxc routines from Adagio
scores.  One way to do this would be to separate the Adagio note-playing
routines into a separate module.  Then users could write their own note
routines that would get parameters from Adagio scores, but that might have
some complex behavior.  For example, one could write a routine that produced
drum@Index(drum rolls) rolls consisting of many MIDI notes, but which could
be invoked by a single line in Adagio.  One would have to be careful to
smoothly integrate Adagio and Moxc; perhaps Adagio should be modified to use
Moxc for its own scheduling.

@section(DX7 Program Librarian)
@index(DX7)
@index(Program Librarian)
In Adagio, one specifies timbre by numbers that correspond to MIDI
programs.  It is up to the user to make sure the proper programs
are loaded into the synthesizer in the right locations.  It would
be nice to be able to write a file like
@begin(example)
Channels  ProgramNumber  ProgramName  
1-8       1              marimba
1         2              harp
2         2              celeste
3-8       2              strings
@end(example)
which would specify a correspondence between program numbers and
program names.  One could write a Program Librarian that could read
such a file and use it to load up a TX816 or whatever with exactly
the right programs for a particular piece.  The program names would
correspond to file names.

@section(An Adagio editor)
@index(editor)
It might be interesting to be able to extract sections of Adagio
scores, stretch them, transpose them, copy them, merge them with
other files, and so on.  This can be done with a text editor, but
a specialized editor, particularly one that could play the files
as it edits them, might be more effective.  Another idea for an
editor is to try to use a combination of typed text and MIDI input
to speed up the process of entering scores.  This is a largely
unexplored area that has applications in music typography systems
as well as CMT.

@section(Interface to XLISP)
An interpretive language like LISP@Index(LISP) can be an excellent teaching
and composing tool.  XLISP@Index(XLISP) is a version of LISP implemented in
C and in the public domain.  It should be fairly simple to provide a LISP
interface to @b(mpu.c), allowing XLISP programs to make music.

@section(A Sequencer)
@index(sequencer)
Everyone else writes sequencers.  Why not build one for CMT?  This would
be the first MIDI sequencer for small machines that users could modify
to suit their special needs.

@section(Ports)
There is very little in CMT that is specific to the IBM PC.  It should not
be too difficult to get CMT running on the Macintosh@Index(Macintosh),
Amiga@Index(Amiga), Atari@Index(Atari), and other small machines.  Work is
under way at Brown University@Index(Brown University) and Carnegie Mellon to
port CMT to an IBM RT@Index(IBM RT) as well.


