@part(adagio, root="cmtman.mss")
@chapter[The Adagio Language]

Adagio @Index(Adagio) is an easy-to-use, non-procedural notation
for scores.   In Adagio, text commands are used to specify each
note.  If you are new to Adagio, you may want to glance at the
examples in Section @ref(adag-examples-sec) starting on page
@pageref(adag-examples-sec) before reading any further.

A note is described in Adagio by a set of attributes@Index(attributes), and
any attribute not specified is ``inherited'' from the previous line.
Attributes may appear in any order and must be separated by one or more
blanks.  An attribute may not contain any blanks.  The attributes are:
time@Index(time), pitch@Index(pitch), loudness@Index(loudness),
voice@Index(voice) number, duration@Index(duration), and
timbre@Index(timbre).

Adagio has been used to program a variety of hardware and software
synthesizers, and the Adagio compiler can be easily adapted to new
environments.  Although not originally intended for MIDI, Adagio works quite
well as a representation for MIDI scores.  The MIDI version of Adagio
currently uses the timbre attribute to select a MIDI ``program''
(synthesizer preset).  Adagio has been extended to allow MIDI controller
data such as modulation wheels, pitch bend, and volume.

A note command in Adagio must be separated from other notes.  Usually,
notes are distinguished by writing each one on a separate line.
Notes can also be separated by using a comma or semicolon as will
be described below.

Besides notes, there are several other types of commands:
@begin(enumerate)
An asterisk@Index(asterisk) (*) in column one (or immediately after a comma
or semicolon) indicates that the rest of the line is a
comment@Index(comment).  The line is ignored by Adagio, and is therefore a
good way to insert text to be read by people.

An empty command (a blank@Index(blank) line, for example) is ignored as if it
were a comment@Index(comment)@foot(To be consistent, a blank line ought to
generate a note that inherits all of its attributes from the previous one.
Adagio is intentionally inconsistent in this respect.  ).

An exclamation point@Index(exclamation point) (!) in column one (or
immediately after a comma or semicolon) indicates a special
command@Index(special command).  A special command does not generate a note,
and currently, the only special commands are the Rate and Tempo commands,
described below.  Special commands extend to the end of the line.

Control change commands are used to control parameters like 
pitch bend that can change as a note is playing.  Control
change commands can be specified along with notes or by
themselves.
A command that specifies only control changes will not produce
a note.
@end(enumerate)

Adagio is insensitive to case@Index(case),
thus ``A'' is equivalent to ``a'', and you
can mix upper and lower case letters freely.

@section[Specifying Attributes]

@subsection[Time]

The time@Index(time) attribute specifies when to start the note.  A time is
specified by a ``T@Index(T)'' followed by a number representing 100@+[th]'s
of a second or by a duration (durations are described below).  Examples:
@begin(programexample)
T150	--1.5 sec
TQ3	--3 quarter note's duration
@end(programexample)
If no time is specified, the default time@Index(default time) is the sum of
the time and duration attributes of the previous note.  (But see Section
@ref(next-time-sec).) Time is measured relative to the time of the
most recent Tempo@Index(Tempo) or Rate@Index(Rate) command.  (See the
examples in Section
@ref(adag-examples-sec) for some clarification of this point.)

@subsection[Pitch]

The pitch@Index(pitch) attribute specifies what frequency to produce.
Standard scale pitches are named by name, using "S" for sharp@Index(sharp),
 "F" for flat@Index(flat),
and (optionally) "N" for natural@Index(natural).
For example, "C" and "CN" represent the same pitch, as do "FS" and "GF" (F
sharp and G flat).

Octaves@Index(octave specification) are specified by
number. C4 is middle C, and B3 is a half step lower.  F5 is the top line of
the treble clef, etc.  Accidentals@Index(accidentals) can go before or after
the octave number, so FS3 and  F3S have the same meaning.  An alternate
notation for pitch is P@i(n), where @i(n) is an integer representing the pitch.
Middle C (C4) is equivalent to P48, CS4 is P49, etc.
@foot{
If you know MIDI codes, you will notice that P48 in Adagio results in
a MIDI pitch code of 60.  The ``middle C = 48'' convention was borrowed
from Gary Nelson's@Index(Nelson, Gary)
MPL@Index(MPL) and it is consistent with MUSIC V@Index(MUSIC V),
cmusic@Index(cmusic), and
many other computer music languages.  I decided to stick with the
convention on the assumption that if you know enough about MIDI to
know that middle C is 60, then you know enough not to be confused by
Adagio and other CMT software.}

If you do not specify an octave@Index(octave specification),
Adagio will choose one for you.  This
is done by picking the octave that will make the current pitch as close
to the previous pitch as possible.  In the case of augmented fourths
or diminished fifths, there are two equally good choices.  Adagio
chooses the lower octave.

@subsection[Duration]

Duration@Index(duration) is specified by a letter indicating a number of
beats, followed by one or several modifiers.  The basic duration codes are:
W@Index(W) (whole, 4 beats), H@Index(H) (half, 2 beats), Q@Index(Q)
(quarter, 1 beat), I@Index(I) (eighth@Index(eighth note), 1/2 beat), and
S@Index(S) (sixteenth, 1/4 beat).  The default tempo is 100 beats per
minute (see Section @ref(tempo-sec)).  These codes may be followed by a "T"
(triplet@Index(triplet)), indicting a duration of 2/3 the normal.  A dot@Index(dot) (".") after a
duration code extends it by half to 3/2 the normal.  Finally, an integer
after a note multiplies its duration by the indicated value (the result is
still just one note).  Examples:
@begin(programexample)
Q	-- 1   beat (quarter note)
QT	-- 2/3 beat (quarter triplet)
W.	-- 6   beats(dotted whole note)
ST6	-- 1   beat (6 sixteenth triplets) 
H5	-- 10  beats(5 half notes)
@end(programexample)
Finally, a duration may be noted by Un@Index(U), where n is an integer
indicating 100@+[th]'s of a second.  For example, U25 is one quarter of one
second.

@subsection(Next Time)
@label(next-time-sec)
The time of the next@Index(next) command (the next command in the Adagio
program text) is
normally the time of the current note command
plus the duration of the current note.
This can be overridden by a field consisting of the letter ``N@Index(N)''
followed by a number indicating hundredths of seconds, or followed by a
duration as described above.  The next note will then start at the time of
the current note plus the duration specified after ``N''.  If the next note
has an explicit time attribute (``T''), then the specified time will override
the one based on the previous note.  Examples:
@begin(programexample)
N0	-- start the next note at the same time as the current one
N50	-- start the next note 0.5 seconds after the current one
NQT	-- start the next note 2/3 beat after the current one
@end(programexample)

@subsection(Rest)
Rests@Index(rests) are obtained by including the field ``R@Index(R)'' in a
note command.  The effect of an ``R'' field is to omit the note that would
otherwise occur as the result of the current note command.  In all other
respects, the command is processed just like any other line.  This means that
attributes such as duration, loudness, and pitch can be specified, and
anything specified will be inherited by the note in the next command.
Normally, a rest will include just ``R'' and a duration.  The fact that a
note command specifies a rest is not inherited.  For example:
@begin(programexample)
R H	-- a half (two beat) rest
RH	-- illegal, R must be separated from other fields by space(s)
@end(programexample)
Because some synthesizers (e.g. a DX7@Index(DX7)) cannot change programs
@Index(program change)
(presets) rapidly, it may be desirable to change programs at the
beginning of a rest so that the synthesizer will be ready to play by
the end of the rest.  See Section @ref(adag-timbre-sec) for an example.

@subsection[Loudness]

Loudness@Index(loudness, velocity) is indicated by an ``L@Index(L)''
followed by a dynamic marking from the following: PPP@Index(PPP),
PP@Index(PP), P@Index(P), MP@Index(MP), MF@Index(MF), F@Index(F),
FF@Index(FF), FFF@Index(FFF).  Alternatively, a number from 0 to 127 may be
used.  The loudness attribute is the MIDI note velocity.  The
dynamic@Index(dynamic markings)
markings are translated into numbers as follows:
@begin(display)
@tabclear
@tabset(0.8 in, 3 in, 3.8 in)
Lppp@\20@\Lmf@\58
Lpp@\26@\Lf@\75
Lp@\34@\Lff@\98
Lmp@\44@\Lfff@\127
@end(display)

@subsection[Voice]

The voice@Index(voice) attribute tells which of the 16 MIDI channels to use
for the note.  The voice attribute consists of a ``V@Index(V)'' followed by
an integer from 1 (the default) to 16.  There is a limit to how many notes
can be played at the same time on a given voice (MIDI channel).  Since the
limit depends upon the synthesizer, Adagio cannot tell you when you exceed
the limit.  Similarly, Adagio cannot tell whether your synthesizer is set up
to respond to a given channel, so there is no guarantee that what you write
will actually be heard.

@subsection[Timbre (MIDI program)]
@label(adag-timbre-sec)
A MIDI program@Index(MIDI program) (synthesizer preset@Index(preset)) can be
selected using the attribute ``Z@i(n)@Index(Z)'', where @i(n)
is the program number.
Notice that in MIDI, changing the program on a given channel will effect
@i(all) notes on that channel.  Also, for many synthesizers, you will not be
able to change programs at the start of a note or during a note.  Change the
program during a rest instead.  For example:
@begin(programexample)
R I Z23 V4	-- change MIDI channel 4 to program 23 during rest
A4		-- play a note on channel 4
@end(programexample)
The cartridge programs on a DX7 can be accessed by adding 32 to the
cartridge program number.  For example, cartridge program number 10
is specified by Z42.

@subsection[Tempo]
@label(tempo-sec)
The length of a beat may be changed using a Tempo@Index(Tempo) command:
@begin(programexample)
!TEMPO @i(n)
@end(programexample)
where @i(n) indicates beats per minute.  The exclamation mark tells Adagio that
this is a special command line rather than a note definition.  A special
command takes the place of a note specification.
No other attributes should be written on a line with a special command.
The new tempo
affects all succeeding notes.  The time@Index(time) attribute of all
succeeding notes is now measured relative to the time of the Tempo command.
Durations specified in hundredths of seconds (for example U58, N15, or T821)
are not affected by the Tempo command.

The Tempo command is fairly clever about default durations@Index(default
durations).  If the last duration specified before the Tempo command is
symbolic (using one of S, I, Q, H, or W), then the default duration for the
node after the Tempo command will be modified according to the tempo change.
Consider the following tempo change:
@begin(programexample)
!TEMPO 60
A4 H
!TEMPO 120
G
@end(programexample)
In this example, the first note will last 2 seconds (2 beats at 60
beats per minute).  The second note inherits the duration (H) from
the first note, but at 120 beats per minute, the second note will last
only 1 second.  If the duration had been specified U200 (also a
duration of 2 seconds), the second note would also last 2 seconds
because the Tempo command does not affect times or durations specified
in hundredths of seconds.

@subsection(Rate)
The rate@Index(rate) command scales all times including those specified in
hundredths of seconds.  A rate of 100 means no change, 200 means twice as
fast, and 50 means half as fast.  For example, to make a piece play 10%
faster, you can add the following command at the beginning of the score:
@begin(programexample)
!RATE 110
@end(programexample)
Rate and Tempo commands combine, so
@begin(programexample)
!RATE 200
!TEMPO 70
@end(programexample)
will play 70 beats per minute at double the normal speed, or 140 beats
per minute.  Like Tempo, the time of the Rate command is added to the
time attribute of all following notes up to the next Time  or Rate
command.

@section[Default Attributes]
If an attribute is omitted, the previous one is used by
default@Index(default) (with the exception of the time attribute).  The
default values for the first note, which are inherited by succeeding notes
until something else is specified, are given below in Adagio notation:
@begin(display)
@tabclear
@tabset(1.5 inch)
Time@\T0
Pitch@\C4
Duration@\Q
Loudness@\LFFF
Voice@\V1
Timbre@\Z1
Tempo@\!TEMPO 100
Rate@\!RATE 100

Pitch Bend@\Y128
After Touch@\O0
Porta Rate@\J99
Porta Switch@\K0
Mod Wheel@\M0
Foot Pedal@\X0
@end(display)

@section[Examples]
@label(adag-examples-sec)
The following plays the first two bars of ``Happy Birthday''.  Note that
Adagio knows nothing of bar lines, so the fact that the first note occurs
on beat 3 or that the meter is three-four is of no consequence:
@begin(programexample)
*Example 1 -- Happy Birthday tune (C major)
!TEMPO 120
G4 I. LF
G4 S
A4 Q
G4
C5
B4 H
@end(programexample)
The time attribute for the first note is zero (0).  The second note
will occur a dotted eighth later, etc. 
Notice that no timbre or rate was specified.
Adagio will provide reasonable default
values of 1 and 100, respectively.

The following example plays the first four bars of  an exercise  from
Bartok@Index(Bartok)'s Mikrokosmos@Index(Mikrokosmos) (Vol.  1, No.  12).
An extra quarter note is inserted at the beginning of each voice in order to
allow time to change MIDI programs.  The right hand part is played on voice
(MIDI channel) 1 and the left hand part on voice 2.  Notice the
specification of the time attribute to indicate that voice 2 starts at time
0.  Also, I have taken advantage of default octaves to reduce typing.
@begin(programexample)
*Example 2 -- Bartok
*voice 1, right hand
R Q Z10 V1
A4 H
B Q
C
D H
C
D Q
C
B
A
B
C
D
R

*voice 2, left hand
T0 R Q Z15 V2
G3 H
F Q
E
D H
E
D Q
E
F
G
F
E
D
R
@end(programexample)

The next example is the same piece expressed in a different manner,
illustrating the interaction
between the Tempo  command and the time attribute.  Recall that the
time attribute is measured relative to the time of the last Tempo command:
@begin(programexample)
*Example 3 -- 4 measures in 2 sections
!Tempo 100
*Voice 1, Measures 1 & 2
R Q Z10 V1
A4 H
B Q
C
D H
C

*Voice 2, Measures 1 & 2
T0 R Q Z15 V2
G3 H
F Q
E
D H
E H

!TEMPO 100
*Voice 1, Measures 3 & 4
T0 V1 D4 Q
C
B
A
B
C
D
R

*Voice 2, Measures 3 & 4
T0 V2 D3 Q
E
F
G
F
E
D
R
@end(programexample)

Notice that the piece is written in 4 sections.  The first
plays a rest followed by two measures, starting
at time 0.  The next section changes the time back to
zero and plays two measures of the left hand part (voice 2).
The next
command (!TEMPO 100) sets the tempo to 100 (it already is)
@i(and) sets the reference time to
be two measures into the piece.  Therefore, the next note
(D4) will begin measure 3.  The D3 that begins the last
group of notes has a T0 attribute, so it will also start at measure
3.  Notice how the !TEMPO command can serve to divide a piece into
sections@Index(sections).


The last example will show yet another way to express the same piece of
music using the ``Next'' attribute.  Only the first bar of music is
given.
@begin(programexample)
*Example 4 -- use of the Next attribute
!Tempo 100
R Q Z10 V1 N0
R Q Z15 V2

A4 H V1 N0
G3   V2

B4 Q V1 N0
F3   V2

C4 Q V1 N0
E3   V2
@end(programexample)
Here, each pair of 
lines represents two simultaneous notes.  The ``N0'' attribute
forces the second line to start at the same time as the first line of each
pair.

@section(Advanced features)
Adagio allows you to write multiple commands
@Index(multiple commands) on one line and to
program MIDI control functions.  It is also possible to write music
with multiple tempi.

@subsection(Multiple notes per line)
Notes can be separated by commas@Index(commas) or
semicolons@Index(semicolon) as well as by starting a new line.  A comma is
equivalent to typing N0 and starting a new line.  A semicolon is equivalent
to starting a new line.  Here is yet another rendition of the Bartok:
@begin(programexample)
*Example 5 -- use of semicolons
!Tempo 100
R Q Z10 V1
A4 H; B Q; C; D H; C; D Q; C; B; A; B; C; D; R

T0 R Q Z15 V2
G3 H; F Q; E; D H; E; D Q; E; F; G; F; E; D; R
@end(programexample)
This example is similar to Example 2, except semicolons are used.
The next example is similar to Example 4, except commas are used
and four bars are notated:
@begin(programexample)
*Example 6 -- use of commas
!Tempo 100
R Q Z10 V1, R Q Z15 V2
A4 H V1, G3 V2
B4 Q V1, F3 V2
C4   V1, E3 V2
D4 H V1, D3 V2
C4   V1, E3 V2
D4 Q V1, D3 V2
C4   V1, E3 V2
B4   V1, F3 V2
A4   V1, G3 V2
B4   V1, F3 V2
C4   V1, E3 V2
D4   V1, D3 V2
R
@end(programexample)

@subsection(Control change commands)
The current version of Adagio can control the following functions:
@begin(display)
Portamento rate@Index(portamento rate) (J@Index(J))
Portamento switch@Index(Portamento switch) (K@Index(K))
Modulation wheel@Index(Modulation wheel) (M@Index(M))
Aftertouch@Index(Aftertouch) (O@Index(O))
Foot pedal@Index(Foot pedal) (X@Index(X))
Pitch bend@Index(Pitch bend) (Y@Index(Y))
@end(display)
The letter listed beside each control function is the Adagio command
letter.  For example, M23 is the command for setting the modulation
wheel to 23.  Except for pitch bend and the portamento switch, all
values range from 0 to 127.  Pitch bend is ``off'' or centered at
128, and has a range from 0 to 255.  Turn on portamento with
K127 and off with K0.

There is no convenient way to get smoothly varying functions.  You
can either type in many control changes to achieve the effect you
want or generate a file with the @i(transcribe) program containing
the data you want.

Control changes can be part of a note specification or independent.
In the following example, a middle C is played with a modulation
wheel setting of 50 and a pitch bend of 120.  Then, at 0.1 second
intervals, the pitch bend is decreased by 10:
@begin(programexample)
*Example 7 -- 
C4 LMF M50 Y120 U100 N10
Y110 N10; Y100 N10; Y90 N10; Y80 N10; Y70 N10; Y60 N10; Y50 N10
@end(programexample)

@subsection(Multiple Tempi)@Index(multiple tempi)
@Index(polyrhythm)
Writing a piece with multiple tempi requires no new commands; you
just have to be clever in the use of Tempo and Time.  The following
plays a 7 note diatonic scale on voice 1, and a 12 note chromatic
scale on voice 2:
@begin(programexample)
*Example 8 -- multiple tempi
!TEMPO 70
V1 C4; D; E; F; G; A; B
T0 R N0

!TEMPO 120
V2 C4; CS; D; DS; E; F; FS; G; GS; A; AS; B

!TEMPO 100
V1 C5, V2 C5
@end(programexample)
The third line plays the 7-note diatonic scale on voice 1.  The
next line contains the tricky part:  notice that the time is
set back to zero, there is a rest, and a next (N) attribute is used
to specify that the next default time will be at the same time as
the current one.  Thus, the time of the !TEMPO 120 command is zero.
After the 12 note scale, the tempo is changed to 100 and a final note
is played on each voice.  A little arithmetic will show that 7 notes
at tempo 70 and 12 notes at tempo 120 each take 6 seconds, so the
final notes (C5) of each scale will happen at the same time.

@section(Running Adagio)
@label(adag-resp-sec)
To use Adagio, create an Adagio score using a text editor.  By convention,
Adagio scores are named @i(something).gio, where @i(something) is any
name you like, and ``.gio'' is a hint that this is an Adagio score.

To listen to the score, type
@begin(example)
adagio @i(something)
@end(example)
Notice that typing the ``.gio'' is optional with Adagio (but you 
must type the full file name when you use the editor).
If all goes well, you will get the following prompt:
@begin(programexample)
Type <RETURN> to play notes, q<RETURN> to quit, ?<RETURN> for help:
@end(programexample)

If you type ``?@c(return)'', you get a complete list of commands:
@begin(programexample)
  <return>       Play music
 q<return>       Quit Adagio (Exit)

 r<return>       Read tuning file
 m<return>       Turn on MIDI byte trace
-m<return>       Turn off MIDI byte trace
 s<return>       Report state
 t<return>       Turn on music operation trace
-t<return>       Turn off music operation trace
 ?<return>       This message
@end(programexample)

If you
type @c(return) to start your piece, Adagio first sends a MIDI program
change to program 1 on each of the 16 MIDI channels.  Exactly 1 second after
typing @c(return), Adagio starts playing your score.

If you type ``q@c(return)'', Adagio will exit.

If you type ``r@c(return)'', you will get a prompt for a tuning file
(see Chapter @ref(tuning-chap)).  This command is useful for playing
a single score with different tunings without exiting Adagio.

The ``m'' command causes Adagio to print a trace of all MIDI data.
This is not useful for anything other than debugging changes to Adagio.
The ``-m'' command turns tracing off.

The ``s'' command reports what data will be traced.

The ``t'' command causes Adagio to print a trace of all music operations.
The information printed in a fairly readable form as opposed to the
numbers printed using the ``m'' switch.
The ``-t'' switch turns off the music operation trace.

@subsection(Debugging Adagio Scores)
@Index(debugging)
While the score is playing, you can type the space bar@Index(space bar) to
stop the performance.  The computer screen will then tell you where it was
at the moment you hit the space bar.  This is very handy for finding the
source of bad notes.  For example, if you hear a bad note, play the score
again and type a space when you hear the mistake@Index(mistake).
Suppose the screen says
that you were playing a note from line 123 and another from line 259.
Remember these numbers, quit Adagio (type ``Q''), and type
@begin(example)
edit @i(something).gio
@end(example)
Now type (to the editor@foot(I assume you have an emacs-like editor.))
@c(ctrl)-U 1 2 2 @c(ctrl)-N.  This moves the cursor down 122 lines to line
number 123.  If the mistake is not found, put the cursor back at the top of
the file (ESC <) and go down 258 lines (@c(ctrl)-U 2 5 8 @c(ctrl)-N) to get
to line 259.

@subsection(Command Line Options)
The following options can be specified in the command line:
@begin(itemize)
-debug (or -d): turn on debugging code.

-help: print a list of command line options.

-miditrace (or -m): turn on MIDI byte trace.

-trace (or -t): turn on music operation trace.

-at: tell Adagio that you are running on an AT.  (This option is
redundant information for debugging.  There is no advantage to using
it because Adagio and other CMT software can determine the machine
type automatically.)

-metronome (or -met): turn on the MPU-401 metronome.  This is only useful for
debugging because there is no way to set the metronome speed.

-simulated (or -s): simulate the MPU-401 but do not use it.  This is
only useful for debugging and may not be implemented.

-xt: tell Adagio that you are running on an XT.  (This option is 
for debugging _ see -at.)

-print: tells Adagio to print details of each note on the console.

-block: turns off the MPU-401 MIDI THRU function so that the input
is not passed to the output.

-tune @i(filename): tells Adagio to use @i(filename) to specify
tuning.
@end(itemize)

@subsection(Examples)
Run Adagio with @i(score.gio) as the score:
@begin(programexample)
adagio score
@end(programexample)

Run Adagio with @i(score.gio) as the score and
using @i(meantone.tun) to specify tuning:
@begin(programexample)
adagio score -tune meantone
@end(programexample)

