@part(dxget, root="cmtman.mss")
@chapter(DXGet and DXPut)

DXGet@Index(DXGet) and DXPut@Index(DXPut) are used to store and recall
programs for synthesizers.  These are generally voice programs, but in
general they can be any sequence of MIDI system exclusive messages.
@Index(system exclusive messages)

@section(DXGet)
The DXGet program is used to record MIDI system exclusive messages and
store them in files.  The program is designed to work with DX7 and
TX816 synthesizers, although it should work with any others.
To run DXGet, type
@begin(programexample)
dxget @i(filename)
@end(programexample)
where @i(filename) is the name of a file you want to create.  If you
leave off the extension, DXGet will use the extension ``.dx7''.  If you
do not specify a file, you will be prompted for one.
After DXGet starts, you will get the following prompt:
@begin(programexample)
Do you want instructions? [y]
@end(programexample)
to which you can respond with "y," "n," or just @c(return) to indicate
yes.  If your response is yes, the following instructions are printed:
@begin(quotation)
This program will let you save DX7 and TX816 voice programs.
To save DX7 programs, you must connect
DX7 MIDI OUT to MPU 401 MIDI IN.
For TX816 programs, connect TX816 MIDI OUT to MPU 401 MIDI IN,
and set the OUT SLOT (leftmost display) to the module
from which you want data.
For DX7: push the orange FUNCTION key, then the green 8.  Push 8
repeatedly until the display says SYS INFO AVAIL or
SYS INFO UNAVAIL.  If the display says SYS INFO UNAVAIL,
push YES to change the display to SYS INFO AVAIL.
Now, select a voice with the INTERNAL or CARTRIDGE button
followed by a (green) number.
For ALL DX7 voices: after getting the SYS INFO AVAIL display,
push 8 again to get MIDI TRANSMIT ?.  Push YES.
For TX816: Hold down SELECT until you see UT in the display.
Let up on SELECT and then push it once more to get the
flashing dU.  Push YES to dump all data from this module.
Hold down SELECT until you see digits in the display.
After data is dumped, dxget will tell you what it got and write
data to a file.  Use dxput to copy a file to a dx module.
@end(Quotation)

Next, you are prompted as follows:
@begin(programexample)
Ready for your data. Type space when you are done...
@end(programexample)
and you should cause your synthesizer to send whatever system
exclusive messages you want to save.  One or more messages can be
saved.  Notice that DXGet will not request information automatically.
This would require DXGet to know about types of synthesizers.  To be
more general, DXGet is very simple.  Consequently, you must manually
cause your synthesizer to send the desired information to DXGet.

After one or more messages have been sent, type the space bar and the
messages will be saved in the file you specified.  The file format is
simply bytes in hexadecimal separated by spaces.
As the data is output, DXGet will check to make sure the data has
not been garbled.  A warning will be printed if any errors are detected.
DXGet will also try to print a description of the exclusive messages
it received.  You can send the data
back to your synthesizer using the DXPut program described below.

@i(NOTE:) If you find that you are sending two messages instead of one
when you use DXGet, here is what is probably happening:  When you push
the ``yes'' button to send data, the DX7 sends a MIDI message that means
``push the `yes' button'', analogous to sending a MIDI note-on message
whey you press a key.  Next, the DX7 sends voice data as requested.
Meanwhile, the first MIDI message has gone into the MPU-401 MIDI IN
port and has been forwarded to the MIDI OUT port.  If the MPU-401 MIDI
OUT is connected to the DX7 MIDI IN, then the DX7 gets a second request
(via MIDI) to send data!  You can avoid this by disconnecting the DX7
MIDI IN cable or by using the ``-block'' switch when you run DXGet.

@section(DXPut)
The DXPut program takes files created by DXGet and sends them as MIDI
output.  To run DXPut, type 
@begin(programexample)
dxput @i(filename)
@end(programexample)
where @i(filename) is the file created by DXGet.  A default extension
of ``dx7'' is assumed if you do not specify one.  After starting, DXPut
will ask:
@begin(programexample)
Do you want instructions? [y]
@end(programexample)
The instructions are as follows:
@begin(quotation)
This program will let you send voice programs to a DX7 or TX816.
You must send a file previously created by the dxget program.
To send DX7 programs, you must connect
MPU 401 MIDI OUT to DX7 MIDI IN.
For TX816 programs, connect MPU 401 MIDI OUT to TX816 MIDI IN.
For a DX7 voice: push the orange FUNCTION key, then the green 8.
Push 8 repeatedly until the display says SYS INFO AVAIL
or SYS INFO UNAVAIL.  If the display says SYS INFO
UNAVAIL, push YES to change to SYS INFO AVAIL.
Next, push MEMORY PROTECT/INTERNAL, then NO, and then
then MEMORY SELECT/INTERNAL. Now, type the name of a file
created by dxget and then the space bar to send data.
To store the voice program on the DX7, hold down STORE
while you push a (green) number.  The current voice at
this number will be erased and replaced by the new voice.
For ALL DX7 voices: Same as for a single DX7 voice, but all 32
voice programs on the DX7 will be replaced when you type
Y in response to `Are you sure?' on the computer.
For all TX816 voices: To store into one module, push NO until the
MEMORY PROTECT light goes out.  Make sure other MEMORY
PROTECT lights are ON.  Type Y in response to 'Are you
sure?' on the computer.
@end(quotation)

Assuming the file you specified contains valid data, you will next
see the prompt:
@begin(programexample)
Ready with your data. Type space when you are ready...
@end(programexample)
Type space to send the data.  If a message is one which DXPut knows
will cause a major change, you will be asked for confirmation before
the data is sent.  At present DXPut only asks you to confirm
Yamaha 32 voice and 64 performance data messages.

After the data in the file is sent, you are prompted by:
@begin(programexample)
Type Q to quit, RETURN to send another file:
@end(programexample)
As the computer says, you can quit or continue to send more data.  Typing
a file name gets you back to the prompt: ``Do you want instructions? [y]'',
which even if you asked for them earlier have
by now have scrolled off the screen.

