@part(intro, root="cmtman.mss")
@begin(titlepage)
@begin(titlebox)
@blankspace(0.5 inch)
@majorheading(The CMU MIDI Toolkit)
@b(``Charles Ives'' Edition)
@blankspace(0.3 inch)
@b(Roger B. Dannenberg)
@value(date)
@end(titlebox)
@center(Carnegie Mellon University)
@center(Pittsburgh, PA 15213, U.S.A.)
@blankspace(1 inch)
@include(disclaimer.mss)
@end(titlepage)
@pageheading(even, left "Page @Value(Page)", right "@c(CMU MIDI Toolkit)")
@pageheading(odd, left "@c[@title(chapter)]", right "Page @Value(page)")
@prefacesection(Preface)
This manual is a guide for users of the Carnegie Mellon University
MIDI Toolkit, also known as CMT,
a collection of software for experimental computer
music using standard Musical Instrument Digital Interface (MIDI@Index(MIDI))
equipment.  This manual corresponds to CMT Version 2.00.

This is the Charles Ives edition, the first CMT manual.  This manual is
updated periodically as new programs are contributed to CMT and as bugs are
discovered and fixed.  I solicit your help in making this manual more
accurate and complete.  PLEASE help us by noting any errors@Index(errors),
omissions@Index(omissions), or suggestions@Index(suggestions) you may have.
You can send your suggestions to Dannenberg@@A.CS.CMU.EDU (ARPAnet) via
computer mail, or by campus mail to Roger B. Dannenberg, Center for Art and
Technology, CFA, or by ordinary mail to Roger B. Dannenberg, Center for Art
and Technology, Carnegie Mellon University, Pittsburgh, PA 15213-3890, USA.
A bug report form can be found at the end of this manual.

Many people have contributed to CMT which grew out of earlier computer
music systems used in the CMU Computer and Electronic Music Studio.
Dean Rubine wrote the first MPU-401 interface routines with help from
Dale Amon.  Joe Newcomer ported this first version to his IBM-AT and
helped clear up many of our interrupt problems.  Aaron Wohl wrote
routines to use the PC timer and made many good suggestions for improvements.
The Tuning program and the initial tuning software were written by Michael
Cressman.  
Most of the development work was done on the IBM-XT in the CMU Computer
and Electronic Music Studio.  This machine was part of a grant from IBM.

I also wish to acknowledge support from CMU through the Music Department,
the Computer Science Department, the Center for the Design of Educational
Computing, and especially the Center for Art and Technology.

@chapter(Introduction and Overview)

The @i(CMU MIDI Toolkit) (CMT) is a software package designed for
experimental computer music education, composition, performance, and
research.  CMT includes a compiler for a simple, text-based music
language, and software for recording, playing, and computing MIDI data
in real time.  CMT has three major attractions:  the flexibility of an
open-ended design, the availability of source code, and low system
cost.

What does CMT do?  The major components and their functions are:
@begin(itemize)
@b(Adagio) is a language and an associated compiler.  In Adagio, a
note is represented by a line of text that specifies attributes such
as pitch, duration, and loudness.  Adagio is quite flexible and is
compatible with several different ways of thinking about scores.  For
example, ``Q'' stands for a quarter note, but duration can also be
indicated by ``U87'', which means 0.87 seconds.  Adagio also supports
arbitrary tuning systems.

@b(Transcribe) reads input from a MIDI keyboard or other MIDI
controller and produces an Adagio score.  The score can be played back
using Adagio, or it can be edited with a text editor to make
alterations.  Since Transcribe output is at least somewhat readable by
people, it can also be used for performance analysis.

@b(Record) is a combination of Adagio and Transcribe.  It plays an
existing Adagio score while transcribing a performance.
Thus, complex compositions can be created one part at a time as if
using a multi-track tape recorder.

@b(DXGet) and @b(DXPut) are programs for recording and replaying MIDI
system exclusive messages.  These programs are typically used to save
and restore synthesis parameters for a digital synthesizer.

@b(Moxc) is a real-time programming environment that is ideal for
writing
interactive music programs.  Moxc is an extension of the C programming
language and is based on Douglas Collinge's @i(Moxie) language.

Also provided are routines (in C) that allow direct production of MIDI
output.  Other routines are available to read MIDI data from a circular
input buffer and to get the current time with 0.01 second resolution.
@end(itemize)

@b(Required hardware and software.)  CMT runs on an IBM-XT or IBM-AT
with a Roland MPU-401 MIDI interface and IBM-PC interface adapter.
You should also have a screen-based text editor.
To use Moxc, you will need a Lattice C compiler.

@b(Other details.)  CMT is distributed by the CMU Center for Art
and
Technology, Carnegie-Mellon University, Pittsburgh, PA, 15213.
We hope that users will contribute new software to the system
and enhance the existing software in various ways.  We will 
encourage this by helping to integrate and document new software
and by distributing software updates to CMT users.

@section(Installing CMT)
CMT comes on a set of two floppy disks.  Disk #1 contains executable
programs and some example files.  Disk #2 contains source code and
files you need in order to recompile CMT programs.
If you have a hard disk, you should copy the contents of the floppy disks
that come with CMT onto the hard disk.  Put Disk #1 in the floppy disk
drive and type:
@begin(example)
c:
cd \
mkdir cmt
cd cmt
copy a:*.*
@end(example)
There are some test files in the directory @i(test) on Disk #1.  You can
copy them as well by typing:
@begin(example)
mkdir test
cd test
copy a:\test\*.*
@end(example)
Some useful non-music programs are included with CMT.  They are
@i(make), @i(grep), and @i(ls).  To install these programs, type:
@begin(example)
cd \bin
copy a:\bin\*.*
@end(example)
If you want to use CMT to write programs, you should copy the contents
of Disk #2 into your @i(cmt) directory.  Put Disk #2 into the floppy disk
drive and type:
@begin(example)
cd \cmt
copy a:*.*
@end(example)
Finally, you add @i(c:\cmt) to the path in your @i(autoexec.bat) file so
that DOS can find CMT programs.

Assuming you have installed an MPU-401 MIDI interface, you should now
connect your synthesizer(s).  At Carnegie Mellon, we generally use
a Yamaha DX7 as input and a TX816 as output.  Thus, we connect the
DX7 MIDI OUT to the MPU-401 MIDI IN and the MPU-401 MIDI OUT to the
TX816 MIDI IN.  @i(You should not connect the MPU-401 OUT to the DX7
IN.)  Doing so will cause outputs from the DX7 to be returned as inputs.
This can cause problems in the DX7.  

If you want to use the same synthesizer for both input and output, a
slightly different approach is necessary.  Here, you should go ahead and
connect both MPU-401 IN and OUT to your synthesizer OUT and IN, respectively.
To avoid the problem of having outputs returned as inputs, you should use
the ``-block'' option on all CMT software.  When the software is not
running, it may be necessary to disconnect either of the two MIDI cables
because the MPU-401 default is to resend all inputs as outputs.
In some cases, the DX7 cannot handle having its output data returned to its
input.
