@part(moxc, root="cmtman.mss")
@chapter(Moxc: real-time programming package)
@begin(verse)
It is the cause, it is the cause, my soul:
Let me not name it to you, you chaste stars!
It is the cause.@!
@/    Othello V.2
@end(verse)
@section(What is Moxc?)
Moxc@Index(Moxc) is a set of C procedures to help the programmer
write programs that perform many tasks simultaneously.
The most important procedure is called @i(cause@Index(cause)), which
allows you to make things happen not now, but in the future@Index(future).
Moxc is a compatible extension of all the programming constructs you
learned in the last chapter.

Moxc is derived from a system called Moxie@Index(Moxie), written by Douglas
Collinge@Index(Collinge, Douglas) of the University of Victoria.  The
original Moxie runs on a Synclavier I, and another version exists as an
extension to the Forth language.

@section(An example)
The following example uses Moxc to implement a simple performance
instrument.  A blow-by-blow commentary follows the example:
@begin(programexample)
@include(moxcexample.c)
@end(programexample)

The first line is an optional comment which is ignored by the compiler.
The next line of text tells the compiler to @i(include) some definitions
from a file called @i(\cmt\musiprog.h) as described in the last chapter.

The procedure @i(keydown@Index(keydown)) is called automatically by Moxc
whenever a keyboard key is pressed.  Three parameters, the MIDI channel, the
key number, and the key velocity are passed to @i(keydown).  When a key is
pressed, @i(k) will have a value where 48 corresponds to middle C, 49 is
C-sharp, and so on.

The procedure @i(pnote) takes two parameters: @i(pitch) and @i(duration),
and it is the same procedure described in the last chapter (see Section
@ref(pnote-sec).  Notice how I added 12 to @i(k) in the first parameter
position to make the @i(pitch) always one octave higher than the key that
was pressed.  The second parameter is always 100, for a duration of one
second.

The next procedure definition is for @i(asciievent@Index(asciievent)).  This
procedure is called whenever a character is typed at the computer.  The
character typed is passed as a parameter to @i(asciievent).  To denote a
character@Index(character value) value in C, you enclose the character in
single quotes as shown in the example.  The @i(asciievent) in the example
only responds to one letter, 'q', which results in a call to @i(quit), a
built-in procedure that tells Moxc to halt@Index(halt) execution.
Notice that 'q' is written in single quotes to denote the letter
``q''.  Without quotes, @i(q) would refer to a parameter or procedure
name.  The exact
same @i(asciievent) procedure is provided by default if you do not provide
your own.

With just these two procedures, a simple instrument has been
implemented which doubles every note up one octave, and holds the
octave for one second.  When using the DX7 and TX816, everything
you play on the DX7 will be sent directly to the TX816 as well as to
Moxc.  Thus, you always hear what is being played on the DX7
combined with any notes played by your Moxc program.  (Unless you
use the -block option in the command line.)

Moxc provides a few more features.  Look at the next procedure definition,
for @i(hello).  The @i(hello) procedure prints ``hello\n'' on the terminal.
(The ``\n'' code at the end means type a newline, putting the cursor at the
beginning of the next line.) The next line is the interesting one:
@i[cause@Index(cause)(300, hello);] means that Moxc should, after 300 time
units, cause the procedure @i(hello) to be called.  In other words, travel 3
seconds into the future, call @i(hello), and return to the present.  In
effect, we have realized time@Index(time travel) travel in the computer!  If
this makes you uncomfortable, what really happens is that Moxc saves a
record of the procedure to be called, the time to call it, and any
parameters that should be passed to it.  When the time comes, the call is
made automatically by Moxc.  In the case of @i(hello), the effect is that
every 3 seconds, ``hello'' is printed on the screen, and the performer may
be simultaneously playing the keyboard.  Why does @i(hello) print ``hello''
repeatedly?  Because after each printing, @i(hello) uses @i(cause) to
schedule another call to @i(hello) in the future.  This call will schedule
yet another call, and so on.

How does @i(hello) ever get started?  It won't, unless someone calls it
for the first time.   This is done in @i(mainscore), which is always
called  once at the beginning of the program, and therefore must be
present in your program.

@section(Compiling)
@Index(compiling)
There are three simple steps to take you from a text file (created with
the editor) to a running program.  Let us assume your file is called
@i(test.c):
@begin(enumerate)
@i(Compile) your program by typing @i(cmtc test.c).  Any detected
errors will be 
listed on the screen and must be corrected before the next step.

@i(Link) your program by typing @i(cmtl test), without the ``@i(.c)''!

@i(Run) your program by typing @i(test).
@end(enumerate)

That's all there is to it.  You must repeat all three steps each time
you edit your file.

@section(More about @i(cause))
What would have happened if @i(mainscore) (above) would have called
@i(hello) twice?  Would the characters ``h e l l o'' get scrambled
together?  Would ``hello'' be printed once or twice every three seconds?
Would the computer get confused and blow up?  In fact, what happens in
Moxc is that every procedure runs to completion before any other
procedure is called.  If a procedure calls another directly, such as
@i(mainscore) calling @i(hello), then the caller is suspended as usual
until the callee completes.  Thus, if there were two calls to @i(hello),
they would execute in sequence, and letters would not be scrambled.

Now, each of the calls to @i(hello) would also issue a call to @i(cause),
so Moxc would have two records telling it to call @i(hello) again at
time 300.  The Moxc system picks one of the two records at time 300
and calls @i(hello).  When this instance of @i(hello) completes, Moxc
notices that there is another record for a call to @i(hello) at time 300
and makes that call too.  By this time, Moxc has probably fallen behind
schedule.  In general, Moxc will call procedures as soon @i(after) the
indicated time as possible.  This time will be quite accurate unless
something is keeping the computer very busy.

The current version of Moxc will quit automatically when it has nothing
scheduled to do.  It will then prompt you to see if you want to run your
program again.  This can be rather annoying if you plan to give it something
to do in the future, say by playing a note on a keyboard.  The program
example shows how a @i(hello) procedure can be used to make sure that Moxc
is always waiting to do something (call @i(hello)), thus being content to
run forever.  Also, I find it helpful when debugging@Index(debugging) to be
able to look at the screen and know Moxc is running normally, as indicated
by the hello messages.  The other way to halt Moxc is to call the @i(quit)
procedure@foot(You can also type @c(control)-C or
@c(control)-@c(break) to
kill almost any process.).  This will
abort any requests to @i(cause) procedure calls.

You can pass parameters@Index(parameters) through @i(cause) to a future
procedure call by listing up to 8 of them after the procedure name.  Here is
a version of @i(hello) that prints the number of times it has been called:
@begin(programexample)
hello(i)
    begin
	printf("hello: %d\n", i);
	cause(300, hello, i+1);
    end
@end(programexample)
On each call to @i(hello), @i(i) is created anew with a value that is one
greater than the value of @i(i) in the previous call to @i(hello).  To print
the  value of @i(i), the characters "%d" are placed anywhere within the
first (string) argument to @i(printf), and the number to be printed is
passed as the second argument.  The procedure @i(printf) will replace %d by
a decimal representation of the number as it prints the string.  If you do
not understand this last example (it is not simple!), you will probably
have trouble writing Moxc programs, so please ask your instructor for a more
detailed explanation.

The @i(cause) procedure does not use a fast algorithm for scheduling
future events.  In fact, @i(cause) does not really schedule anything; it
merely puts events on a list of events to be scheduled at some later time.
For every waiting event Moxc runs, Moxc takes at least one
event from the list of
events created by @i(cause) and inserts it into another list of events
that are waiting to run.  This waiting list is ordered by event time.
If there are hundreds of events in the list and Moxc has to insert an
event at the end (because the event happens later than the others), Moxc
may delay a noticeable amount of time.  So far, this behavior has been
rather interesting, so no attempts have been made to make Moxc faster
or more accurate under heavy loads.

@section(Event Handlers)
Event@Index(event handler) handling procedures were described in the example
above, including
@i(keydown) and @i(asciievent).  A complete list of these is given below.
If your program does not provide an event handler, the @i(cmtl) (link)
step of preparing your program will provide a default handler which does
nothing except for @i(asciievent), which is given in the example above.
@begin(description)
asciievent@Index(asciievent)(k)@\action for terminal input, @i(k) is
the ascii key code.

bendchange@Index(bendchange)(ch, val)@\pitch bend handler; @i(ch) is
the channel, and @i(val) is the new value.

ctrlchange@Index(ctrlchange)(ch, c, val)@\control change handler;
@i(ch) is the channel, @i(c) is the control number, and @i(val) is the new
value.

keydown@Index(keydown)(ch, p, v)@\MIDI note on handler; @i(ch) is the
channel, @i(p) is the pitch code (48 = middle C), and @i(v) is the velocity.

keyup@Index(keyup)(ch, p)@\MIDI note off handler; @i(ch) is the
channel, and @i(p) is the pitch code..

mainscore@Index(mainscore)()@\first action(s).  This ``event'' is
called once at the beginning of execution.

peddown@Index(peddown)(ch)@\sustain pedal down handler; @i(ch) is the
channel.

pedup@Index(pedup)(ch)@\sustain pedal up handler; @i(ch) is the channel.

touchchange@Index(touchchange)(ch, val)@\aftertouch handler; @i(ch) is
the channel, and @i(val) is the value.
@end(description)


@section(Command Line Options)
The following options can be specified in the command line:
@begin(itemize)
-debug (or -d): turn on debugging code.

-help: print a list of command line options.

-miditrace (or -m): turn on MIDI byte trace.

-trace (or -t): turn on music operation trace.

-block: turns off the MPU-401 MIDI THRU function so that the input
is not passed to the output.

-tune @i(filename): tells Adagio to use @i(filename) to specify
tuning.
@end(itemize)

@subsection(Examples)
Run Moxc program named @i(echoes)@foot(Note that @i(echo) is a special DOS
command, so that is why I named this program @i(echoes).  The @i(echoes)
program and its source code are included on the CMT disks.):
@begin(example)
echoes
@end(example)

Run the @i(echoes) program with tuning file @i(scale23.tun):
@begin(example)
echoes -tune scale23
@end(example)

