@part(record, root="cmtman.mss")
@chapter(The Record Program)
@section(Overview)
The @i(Record@Index(Record)) program is a combination of Adagio and
Transcribe.  It is used to record performances via MIDI and to translate
them into Adagio @i(while simultaneously playing an Adagio score.)  This
allows you to enter scores one part at a time.  Section @ref(splice-sec)
discusses how you can combine several Adagio scores into one.

@section(Instructions)
Before using Record, you should first create an Adagio score, either
with an editor or with Transcribe.  This will be the score that Record
plays while recording a new score.  Let us assume that the existing score is
called @i(old.gio) and you want to record another part called @i(new.gio).
Run Record by typing:
@begin(example)
record old
@end(example)
Record uses the Adagio compiler to get @i(old.gio) ready to play.  It
then prompts you with:
@begin(example)
Type <RETURN> to record, q<RETURN> to quit, ?<RETURN> for help:
@end(example)
The possible responses are described in detail in Section @ref(adag-resp-sec).
Normally, you should just type @c(return) to start playing and recording.
Next, you will get a prompt for the file you want to create:
@begin(example)
Name of output file:
@end(example)
Type @i(new).  The extension @i(.gio) will be added automatically if you
do not provide any extension.

After a delay of one second, Record begins to play the score in @i(old).
The delay is not precise, so you would normally put something into @i(old)
to ``count off'' the start of the piece.  Record also records anything you
play.  @i(Recording does not stop until you type the space bar), even if
@i(old) finishes playing.

After you type the space bar@Index(space bar), you will again get the prompt:
@begin(example)
Type <RETURN> to record, q<RETURN> to quit, ?<RETURN> for help:
@end(example)
You can erase and re-record @i(new) by repeating the instructions given
above.  (This time, you will have to confirm that you want to erase
@i(new.gio).)  Otherwise, just type @i(q) to quit.

You now have two files, @i(old.gio) and @i(new.gio).  The next section
describes how to merge files to create a single file for Adagio.

@section(Merging Adagio Scores)
@label(splice-sec)
@index(merging two scores)
A few tricks are helpful in merging two scores into one.  In this section,
I will describe the basics of making two scores play at the same time
as well as getting two scores to play in sequence.  It should be remembered
that the goal is simply to create an Adagio score.  If you try something that
does not work, you should be able to look at your scores and figure out
where the problem is.

@subsection(Playing two scores in sequence)

To play two scores in sequence, you should edit the first score
file so that the last line contains the last event of
that score.  Since Adagio scores are not necessarily written in time
order, this may or may not be the case.  Transcribe and Record always
write scores in the order of note start times.  The last note will
specify a next (N) attribute that corresponds to the time you typed
the space bar.

Once you get the timing of the last note right, it should be apparent
that if you insert some notes at the end of the score, they will
be played after the score (unless they specify a time (T) attribute).
You might want to conduct an experiment to find out where your score
ends.  If you insert the line
@begin(example)
C6 LFFF U20
@end(example)
at the end of your file, you should here a distinctively loud and high
note at the end of your score.  If the ``end'' comes too early or too
late, you should edit your file accordingly.  

You can insert@Index(insert)@index(append) an entire file at the end of
another file using the DOS copy@Index(copy) command:
@begin(example)
copy score1.gio+score2.gio
@end(example)
DOS knows nothing about Adagio, so the @i(.gio) extensions must be
specified.  This command will modify @i(score1.gio) by appending the contents
of @i(score2.gio), which is not modified.  An alternative form is
@begin(example)
copy score1.gio+score2.gio score3.gio
@end(example)
which puts the result into @i(score3.gio), leaving @i(score1.gio) and
@i(score2.gio) intact.

If @i(score2.gio) does not specify (T) attributes, then appending
the scores should be adequate.  On the other hand, if @i(score2.gio)
specifies times relative to the beginning of the score, you can
insert a !TEMPO command at the beginning.  Then, when @i(score2.gio)
is appended to @i(score1.gio), the !TEMPO command will occur at the
end of @i(score1.gio), and everything in @i(score2.gio) will be
measured relative to the !TEMPO command.

@subsection(Playing two scores at the same time)
To get two scores to play at the same time, you should make sure that
the second score includes a time (T) attribute on or before its first
note.  Otherwise, the second score will be played after the first one.
(Record automatically inserts a time attribute on the first note of
each file it creates.  The time is the correct one assuming you want
to merge the recorded file (e.g. @i(new.gio) with the played file (e.g.
@i(old.gio)).  The DOS copy command is used to merge the two files as
described above.

A typical problem is trying to play two scores on the same voice (MIDI
channel).  This is only a problem if both scores try to play the same note
at the same time.  MIDI assumes that notes are turned on by pressing keys,
and it is impossible to press the same key twice without first releasing it.
This problem usually manifests itself as notes that get cut off early.  If
you are trying to merge two files with the same voice, you might want to
change the voice specified for the second file to make sure the two scores
do not interfere.  (This of course assumes you have several synthesizers so
that you can play several channels at once.)

@section(Command Line Options)
The following options can be specified in the command line:
@begin(itemize)
-debug (or -d): turn on debugging code.

-help: print a list of command line options.

-miditrace (or -m): turn on MIDI byte trace.

-trace (or -t): turn on music operation trace.

-at: tell Record that you are running on an AT.  (This option is
redundant information for debugging.  There is no advantage to using
it because Record and other CMT software can determine the machine
type automatically.)

-metronome (or -met): turn on the MPU-401 metronome.  This is only useful for
debugging because there is no way to set the metronome speed.

-simulated (or -s): simulate the MPU-401 but do not use it.  This is
only useful for debugging and may not be implemented.

-xt: tell Record that you are running on an XT.  (This option is 
for debugging -- see -at.)

-print: tells Record to print details of each compiled note on the console.

-block: turns off the MPU-401 MIDI THRU function so that the input
is not passed to the output.

-control (or -c) on: record control information, do not present the
``Pitch bend, etc. on?'' query.

-control (or -c) off: do not record control information, do not present the
``Pitch bend, etc. on?'' query.
@end(itemize)

@subsection(Examples)
Print information about command line options:
@begin(example)
record -help
@end(example)

Play score in file @i(foo.gio), do not record control information, and
turn off the MIDI THRU function of the MPU-401:
@begin(example)
record foo -c off -block 
@end(example)

